@@ -4,6 +4,7 @@ mod bell;
 pub mod block;
 pub mod call;
 pub mod method;
+pub mod music;
 mod parity;
 pub mod place_not;
 pub mod row;
@@ -17,6 +18,6 @@ pub use call::Call;
 pub use method::Method;
 pub use parity::Parity;
 pub use place_not::{PlaceNot, PnBlock};
-pub use row::{InvalidRowError, Row};
+pub use row::{InvalidRowError, Row, RowBuf};
 pub use stage::{IncompatibleStages, Stage};
 pub use utils::run_len;
