@@ -1,6 +1,6 @@
 //! Module for parsing and handling place notation
 
-use crate::{AnnotBlock, AnnotRow, Bell, IncompatibleStages, Row, Stage};
+use crate::{AnnotBlock, AnnotRow, Bell, IncompatibleStages, Row, RowBuf, Stage};
 use itertools::Itertools;
 use std::{
     fmt::{Display, Formatter},
@@ -73,7 +73,7 @@ pub struct PlaceNot {
 
 impl PlaceNot {
     /// Parse a string, interpreting it as a single `PlaceNot` of a given [`Stage`].  Like
-    /// [`Row::parse_with_stage`], this ignores chars that don't correspond to valid [`Bell`]
+    /// [`RowBuf::parse_with_stage`], this ignores chars that don't correspond to valid [`Bell`]
     /// names, including `&`, `.`, `,` and `+` which have reserved meanings in blocks of place
     /// notation.  This will expand implicit places (even between two written places) but will fail
     /// if there is any kind of ambiguity, returning a [`ParseError`] describing the problem.  This
@@ -81,7 +81,7 @@ impl PlaceNot {
     ///
     /// # Example
     /// ```
-    /// use proj_core::{Stage, PlaceNot, place_not::ParseError};
+    /// use bellframe::{Stage, PlaceNot, place_not::ParseError};
     ///
     /// // Parsing a valid place notation is OK
     /// assert_eq!(PlaceNot::parse("14", Stage::MAJOR)?.to_string(), "14");
@@ -185,14 +185,14 @@ impl PlaceNot {
     ///
     /// # Example
     /// ```
-    /// use proj_core::{PlaceNot, Stage};
+    /// use bellframe::{PlaceNot, Stage};
     ///
     /// // These are crosses
     /// assert_eq!(
     ///     PlaceNot::cross(Stage::MAJOR).unwrap(),
     ///     PlaceNot::parse("x", Stage::MAJOR)?
     /// );
-    /// # Ok::<(), proj_core::place_not::ParseError>(())
+    /// # Ok::<(), bellframe::place_not::ParseError>(())
     /// ```
     pub fn cross(stage: Stage) -> Option<Self> {
         if stage.as_usize() % 2 == 0 {
@@ -209,7 +209,7 @@ impl PlaceNot {
     ///
     /// # Example
     /// ```
-    /// use proj_core::{PlaceNot, Stage};
+    /// use bellframe::{PlaceNot, Stage};
     ///
     /// // These are crosses
     /// assert!(PlaceNot::cross(Stage::MAJOR).unwrap().is_cross());
@@ -217,7 +217,7 @@ impl PlaceNot {
     /// // These are not
     /// assert!(!PlaceNot::parse("14", Stage::MAJOR)?.is_cross());
     /// assert!(!PlaceNot::parse("3", Stage::TRIPLES)?.is_cross());
-    /// # Ok::<(), proj_core::place_not::ParseError>(())
+    /// # Ok::<(), bellframe::place_not::ParseError>(())
     /// ```
     #[inline(always)]
     pub fn is_cross(&self) -> bool {
@@ -271,7 +271,7 @@ impl PlaceNot {
 
     /// Uses this `PlaceNot` to permute a given [`Row`], preserving the old copy and returning a
     /// new [`Row`].  This checks that the [`Stage`]s are equal, and is therefore safe.
-    pub fn permute_new(&self, row: &Row) -> Result<Row, IncompatibleStages> {
+    pub fn permute_new(&self, row: &Row) -> Result<RowBuf, IncompatibleStages> {
         IncompatibleStages::test_err(row.stage(), self.stage)?;
         Ok(unsafe { self.permute_new_unchecked(row) })
     }
@@ -282,8 +282,8 @@ impl PlaceNot {
     /// # Safety
     ///
     /// This function is safe to use only when `self.stage() == row.stage()`.
-    pub unsafe fn permute_new_unchecked(&self, row: &Row) -> Row {
-        let mut new_row = row.clone();
+    pub unsafe fn permute_new_unchecked(&self, row: &Row) -> RowBuf {
+        let mut new_row = row.to_owned();
         self.permute_unchecked(&mut new_row);
         new_row
     }
@@ -572,7 +572,7 @@ impl PnBlock {
     pub fn to_block<A: Default>(&self) -> AnnotBlock<A> {
         // The rows which will make up the new Block
         let mut rows: Vec<AnnotRow<A>> = Vec::with_capacity(self.pns.len() + 1);
-        rows.push(AnnotRow::with_default(Row::rounds(self.stage())));
+        rows.push(AnnotRow::with_default(RowBuf::rounds(self.stage())));
         for pn in &self.pns {
             rows.push(AnnotRow::with_default(unsafe {
                 pn.permute_new_unchecked(&rows.last().unwrap().row())
@@ -747,14 +747,53 @@ mod tests {
 
     #[test]
     fn pn_to_block() {
+        let alnick_lead = "156342
+516324
+153642
+513462
+531426
+354162
+351426
+534162
+354612
+345621
+436512
+463521
+643251
+634215
+362451
+326415
+236145
+321654
+326145
+231654
+213645
+123465
+214356
+124365
+123456";
+        let plain_bob_major_lead = "12345678
+21436587
+24163857
+42618375
+46281735
+64827153
+68472513
+86745231
+87654321
+78563412
+75836142
+57381624
+53718264
+35172846
+31527486
+13254768
+13527486";
+
         let equal_blocks = [
-            (
-                Stage::MINOR,
-                "34-36.14-12-36.14-14.36,12",
-                include_str!("alnwick"),
-            ), // Alnwick Surprise Minor
-            (Stage::MINOR, "34-3.4-2-3.4-4.3,+2", include_str!("alnwick")), // Alnwick Surprise Minor
-            (Stage::MAJOR, "x18x18x18x18,12", include_str!("pb-8")),        // Plain Bob Major
+            (Stage::MINOR, "34-36.14-12-36.14-14.36,12", alnick_lead), // Alnwick Surprise Minor
+            (Stage::MINOR, "34-3.4-2-3.4-4.3,+2", alnick_lead),        // Alnwick Surprise Minor
+            (Stage::MAJOR, "x18x18x18x18,12", plain_bob_major_lead),   // Plain Bob Major
         ];
 
         for &(stage, pn, block) in &equal_blocks {
